import urllib.parse
from unittest import TestCase

from configs.settings.classes import DBSetting


def makeSettings(
    dbtype=...,
    dsn=...,
    dbname=...,
    dbhost=...,
    dbport=...,
    dblogin=...,
    dbpassword=...,
) -> DBSetting:
    """Make db settings for unittests."""
    kwargs = {
        "db_type": dbtype,
        "dsn": dsn,
        "db_host": dbhost,
        "db_port": dbport,
        "db_name": dbname,
        "db_user": dblogin,
        "db_password": dbpassword,
        "db_settings": {"connection_pool_size": 0},
    }
    return DBSetting(settings=kwargs, serviceName="test")


class TestDBSettingsDSN(TestCase):
    def test_no_dsn(self):
        """
        Test no dsn specified.
        """
        settings = makeSettings(
            dsn="",
            dbhost="myhost",
            dbport=42,
            dblogin="mylogin",
            dbpassword="mypassword",
            dbname="myname",
        )
        self.assertEqual(settings.dsn, "myhost:42/myname")
        self.assertEqual(settings.userName, "mylogin")
        self.assertEqual(settings.password, "mypassword")

    def test_dsn_complete(self):
        """
        Test dsn completely specified.
        """
        settings = makeSettings(dsn="mylogin:mypassword@myhost:42/myname")
        self.assertEqual(settings.dsn, "myhost:42/myname")
        self.assertEqual(settings.userName, "mylogin")
        self.assertEqual(settings.password, "mypassword")

    def test_dsn_no_hostspec(self):
        """
        Test dsn with no hostspec specified.
        """
        adaptor = makeSettings(dsn="/myname", dbhost="myhost", dbport=42)
        self.assertEqual(adaptor.dsn, "myhost:42/myname")

    def test_dsn_no_host(self):
        """
        Test dsn with no host specified.
        """
        settings = makeSettings(dsn=":42/myname", dbhost="myhost")
        self.assertEqual(settings.dsn, "myhost:42/myname")

    def test_dsn_no_port(self):
        """
        Test dsn with no port specified.
        """
        settings = makeSettings(dsn="myhost/myname", dbport=42)
        self.assertEqual(settings.dsn, "myhost:42/myname")

    def test_dsn_multiple_hostspec(self):
        """
        Test dsn with multiple hostpec specified.
        """
        adaptor = makeSettings(dsn="myhost:42,myhost2:84/myname")
        self.assertEqual(adaptor.dsn, "myhost:42,myhost2:84/myname")

    def test_dsn_multiple_hostspec_no_host(self):
        """
        Test dsn with multiple hostpec with no host specified.
        """
        adaptor = makeSettings(dsn=":42,:84/myname", dbhost="myhost")
        self.assertEqual(adaptor.dsn, "myhost:42,myhost:84/myname")

    def test_dsn_multiple_hostspec_no_port(self):
        """
        Test dsn with multiple hostpec with no port specified.
        """
        settings = makeSettings(dsn="myhost,myhost2/myname", dbport=42)
        self.assertEqual(settings.dsn, "myhost:42,myhost2:42/myname")

    def test_dsn_no_name(self):
        """
        Test dsn with no db name specified.
        """
        settings = makeSettings(dsn="myhost:42", dbname="myname")
        self.assertEqual(settings.dsn, "myhost:42/myname")

    def test_dsn_query(self):
        """
        Test dsn with some query specified.
        """
        settings = makeSettings(dsn="myhost:42/myname?param=value")
        self.assertEqual(settings.dsn, "myhost:42/myname?param=value")

    def test_dsn_encoded(self):
        """
        Test dsn with encoded values.
        """
        value = "üñ@!"
        valueEncoded = urllib.parse.quote("üñ@!")

        settings = makeSettings(dsn=f"login:{valueEncoded}@myhost:42/myname?param={valueEncoded}")
        self.assertEqual(settings.dsn, f"myhost:42/myname?param={valueEncoded}")
        self.assertEqual(settings.password, value)
